package GrabInfo::Plugin ;

use strict ;
use FindBin ;
use File::Basename ;
use Carp ;
use GrabInfo::XMLTag ;


# La liste des repertoires qu'il faut chercher pour les modules
@GrabInfo::Plugin::INC = ($FindBin::Bin) ;


sub new {
	my $class = shift ;
	my $name = shift ;
	my $context = shift ;
	my $indent = shift ;
	
	my $this = {
		module => $class,
		indent => $indent,
		name => $name,
		context => $context,
		xml_tag_arch => [],
		xml_tag => new GrabInfo::XMLTag($name, $indent),
	} ;
	bless($this, $class) ;

	$this->{path} = $this->module2path($this->{module}) ;

	return $this ;
}


sub run {
	my $this = shift ;

	my $ctx = $this->{context} ;
	my %ctx_cp = %{$ctx} ;
	$this->{context} = \%ctx_cp ;

	$this->{iter_cnt} = 0 ;
	my @iterate = $this->iterate() ;
	if (scalar(@iterate)){
		foreach my $iter (@iterate){
			if ($this->{iter_cnt} > 0){
				push @{$this->{xml_tag_arch}}, $this->{xml_tag} ;
				$this->{xml_tag} = new GrabInfo::XMLTag($this->{name}, $this->{indent}) ;
			}
			eval {
				$this->snapshot($iter) ;
			} ;
			if ($@){
				$this->{xml_tag}->add_error($@) ;
			}

			$this->process_kids() ;
			$this->{iter_cnt}++ ;
	 	}
	}

	$this->{context} = $ctx ;
}


sub set_context_data {
	my $this = shift ;
	my $name = shift ;
	my $data = shift ;

	$this->{context}->{$name} = $data ;
}


sub get_context_data {
	my $this = shift ;
	my $name = shift ;

	return $this->{context}->{$name} ;
}


sub iterate {
	my $this = shift ;

	return (undef) ;
}


sub snapshot {
	my $this = shift ;

	croak("Method snapshot must be implemented for all GrabInfo::Plugins: $this") ;
}


sub process_kids {
	my $this = shift ;
	my $path = shift || $this->{path} ;

	# First we process the modules already loaded in RAM
	foreach my $m (keys %INC){
		if ($m =~ /^$path\/(([^\/]+).pm)$/){
			$this->process_module($path, $1) ;
		}
	}

	# Then we process the ones on disk
	foreach my $I (@GrabInfo::Plugin::INC){
		if (opendir(DIR, "$I/$path/")){
			my @files = grep {$_ =~ /\.(pm|gis)$/} readdir DIR ;
			closedir(DIR) ;

			foreach my $f (@files){
				if (($f =~ /\.pm$/)&&(! exists $INC{"$path/$f"})){
					$this->process_module($path, $f) ;
				}
				elsif (($f =~ /\.gis$/)&&(-x "$I/$path/$f")){
					$this->process_script($I, $path, $f) ;
				}
			}
		}
	}	
}


sub process_module {
	my $this = shift ;
	my $path = shift ;
	my $f = shift ;

	my $bn = basename($f) ;
	my $bn_no_ext = $bn ;
	$bn_no_ext =~ s/\.(pm|gis)$// ;

	my $module = $this->path2module("$path/$bn_no_ext") ;

	eval {
		$this->{xml_tag}->validate_tag_name($bn_no_ext) ;
	} ;
	if ($@){
		next ;
	}

	eval {					
		require "$path/$f" ;
		my $pi = $module->new($bn_no_ext, $this->{context}, $this->{xml_tag}->{indent} + 1) ;
		$pi->run() ;
		if ($pi->{iter_cnt} > 0){
			my $first_tag = (scalar(@{$pi->{xml_tag_arch}}) ? $pi->{xml_tag_arch}->[0] : $pi->{xml_tag}) ;
			$first_tag->set_comment("$module plugin") ;
			foreach my $xta (@{$pi->{xml_tag_arch}}){
				$this->{xml_tag}->add_kid($xta) ;
			}
			$this->{xml_tag}->add_kid($pi->{xml_tag}) ;
		}
	} ;
	if ($@){
		$this->{xml_tag}->add_error($@) ;
	}
}


sub process_script {
	my $this = shift ;
	my $I = shift ;
	my $path = shift ;
	my $f = shift ;

	my $fp = "$I/$path/$f" ;
	my $out = `$fp` ;
	if ($?){
		$this->{xml_tag}->add_error("Can't execute $fp: $? ($out}") ;
	}
	else{
		my $t = $this->{xml_tag}->create_kid("script") ;
		$t->set_comment("script $fp") ;
		my @lines = split(/\n/, $out) ;
		$t->{raw} = \@lines ;
		$this->{xml_tag}->add_kid($t) ;
	}
}


sub run_cmd {
	my $this = shift ;
	my $cmd = shift ;

	my $output = `$cmd 2>&1` ;
	if ($?){
		$this->{xml_tag}->add_error("Can't execute command '$cmd': $? ($output)") ;
		return undef ;
	}
	elsif (! $output){
		$this->{xml_tag}->add_error("$cmd returned no output!") ;
		return undef ;
	}
	else{
		my @lines = split(/\n/, $output) ;
		return \@lines ;
	}
}


sub slurp_file {
	my $this = shift ;
	my $file = shift ;

	if (! open(FILE, "<$file")){
		$this->{xml_tag}->add_error("Can't open file for reading: $!") ;
		return undef ;
	}
	else{
		my @lines = <FILE> ;
		close(FILE) ;
		return \@lines ;
	}
}


my $G = 1024 * 1024 * 1024 ;
my $M = 1024 * 1024 ;
my $K = 1024 ;
sub unitize {
	my $this = shift ;
	my $v = shift ;

	if ($v >= $G){
		$v = sprintf("%.0fG", ($v / $G)) ;
	}
	elsif ($v >= $M){
		$v = sprintf("%.0fM", ($v / $M)) ;
	}
	elsif ($v >= $K){
		$v = sprintf("%.0fK", ($v / $K)) ;
	}

	return $v ;
}


sub path2module {
	my $this = shift ;
	my $path = shift ;
	
	$path =~ s/\//::/g ;
	
	return $path ;
}


sub module2path {
	my $this = shift ;
	my $mod = shift ;
	
	$mod =~ s/::/\//g ;
	
	return $mod ;
}

						
1 ;
