package GrabInfo::Plugin::_machine::_static::disk ;
@ISA = qw(GrabInfo::Plugin) ;

use strict ;
use File::Basename ;


sub snapshot {
	my $this = shift ;

	$this->ide() ;
	$this->scsi() ;
}


sub ide {
	my $this = shift ;

	my $k = $this->{xml_tag}->create_kid("ide") ;
	$this->{xml_tag}->add_kid($k) ;

	my @devices = </proc/ide/hd*> ;
	foreach my $d (@devices){
            my $name = $k->validate_tag_name(basename($d), 1) ;
			my $dk = $k->create_kid($name) ;

			foreach my $f ('capacity', 'media', 'model'){
				if (-r "$d/$f"){
					my $lines = $this->slurp_file("$d/$f") ;
					if ($lines){
						my $data = $lines->[0] ;
						chomp($data) ;
						if ($f eq 'capacity'){
							$data = $this->unitize($data * 512) ;
						}
						$dk->add_attr($f, $data) ;
					}
				}
			}

			$k->add_kid($dk) ;
	}
}


sub scsi {
	my $this = shift ;

	my $k = $this->{xml_tag}->create_kid("scsi") ;
	$this->{xml_tag}->add_kid($k) ;

	if (-r "/proc/scsi/scsi"){
		my $lines = $this->slurp_file("/proc/scsi/scsi") ;
		if ($lines){
			my @l = @{$lines} ;
			# Drop header
			shift @l ;

			my @scsi_devs = $this->get_scsi_devices() ;

			my $nb = 0 ;
			while (scalar(@l)){
				my ($l1, $l2, $l3) = (shift @l, shift @l, shift @l) ;
				my @l1 = split(//, $l1) ;
				my @l2 = split(//, $l2) ;
				my @l3 = split(//, $l3) ;

				my $vendor = join('', @l2[10..17]) ;
				$vendor =~ s/\s+$// ;
				my $model = join('', @l2[26..41]) ;
				$model =~ s/\s+$// ;
				my $type = join('', @l3[10..41]) ;
				$type =~ s/\s+$// ;

				if ($type eq 'Direct-Access'){
					my $device = $scsi_devs[$nb++] ;
					if ($device){
						my $name = $k->validate_tag_name($device->{name}, 1) ;
						my $dk = $k->create_kid($name) ;

						$dk->add_attr('media', 'disk') ;
						$dk->add_attr('model', $model) ;
						$dk->add_attr('vendor', $vendor) ;
						$dk->add_attr('capacity', $this->unitize($device->{'#blocks'} * 1024)) ;
						$k->add_kid($dk) ;
					}
				}
			}
		}
	}
}


sub get_scsi_devices {
	my $this = shift ;

	my @devs = () ;
	my $lines = $this->slurp_file("/proc/partitions") ;
	if ($lines){
		my @header = split(/\s+/, shift @{$lines}) ;
		shift @{$lines} ;

		foreach my $l (@{$lines}){
			$l =~ s/^\s+// ;
			my $cnt = 0 ;
			my %data = map {($header[$cnt++], $_)} split(/\s+/, $l) ;
			if ($data{name} =~ /^sd\w$/){
				push @devs, \%data ;
			}
		}
	}

	return @devs ;
}


1 ;

__END__
Attached devices:
Host: scsi0 Channel: 00 Id: 00 Lun: 00
  Vendor: LSILOGIC Model: 1030 IM          Rev: 1000
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi0 Channel: 00 Id: 08 Lun: 00
  Vendor: IBM      Model: 25P3495a S320  1 Rev: 1
  Type:   Processor                        ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 00 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 01 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 02 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 03 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 04 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 05 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 06 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Direct-Access                    ANSI SCSI revision: 02
Host: scsi1 Channel: 00 Id: 15 Lun: 00
  Vendor: IBM      Model: SERVERAID        Rev: 1.00
  Type:   Processor                        ANSI SCSI revision: 02
Host: scsi1 Channel: 01 Id: 15 Lun: 00
  Vendor: IBM      Model: EXP400   S320    Rev: D110
  Type:   Processor                        ANSI SCSI revision: 03

