package GrabInfo::VServer ;

use strict ;
use GrabInfo::Plugin ;
use Socket ;


# Some global vars...
my $NEW_VSERVER_SYSCALL = 273 ;
my $NEW_VCMD_GET_VERSION = 0 ;
my $NEW_VCMD_S_CONTEXT = 1057030145 ;
my $NEW_VCMD_IPV4ROOT = 1057095683 ;

my $OLD_IPV4ROOT_SYSCALL = undef ;
my $OLD_IPV4ROOT_SYSCALL_VERSION = undef ;
my $OLD_S_CONTEXT_SYSCALL = undef ;

my $CURRENT_S_CONTEXT = 0 ;

my $KERNEL_NONE = 1 ;
my $KERNEL_OLD = 2 ;
my $KERNEL_NEW = 3 ;


my $KERNEL_TYPE = undef ;
init() ;


sub init {
	if (! defined($KERNEL_TYPE)){
		# Grab some info from /proc/self/status
		my $dummy = GrabInfo::Plugin->new('dummy', {}, 0) ;
		my $lines = $dummy->slurp_file("/proc/self/status") ;
		if (! $lines){
			die("Can't read /proc/self/status: $!") ;
		}
		foreach my $l (@{$lines}){
			my @c = split(/\s+/, $l) ;
			if ($c[0] eq '__NR_new_s_context:'){
				$OLD_S_CONTEXT_SYSCALL = $c[1] ;
			}
			elsif ($c[0] eq '__NR_set_ipv4root:'){
				$OLD_IPV4ROOT_SYSCALL = $c[1] ;
				$OLD_IPV4ROOT_SYSCALL_VERSION = $c[2] ;
			}
			elsif ($c[0] eq 's_context:'){
				$CURRENT_S_CONTEXT = $c[1] ;
			}
		}

		# Check for new vserver style
		if (syscall($NEW_VSERVER_SYSCALL, $NEW_VCMD_GET_VERSION, 0, undef) != -1){
			$KERNEL_TYPE = $KERNEL_NEW ;
		}
		else{
			# Check for old vserver style
			if ((defined($OLD_S_CONTEXT_SYSCALL))&&(defined($OLD_IPV4ROOT_SYSCALL))){
				$KERNEL_TYPE = $KERNEL_OLD ;
			}
			else{
				$KERNEL_TYPE = $KERNEL_NONE ;
			}
		}
	}
}


sub supported {
	return (($KERNEL_TYPE == $KERNEL_OLD)||($KERNEL_TYPE == $KERNEL_NEW)) ;
}


sub get_current_ctx {
	return $CURRENT_S_CONTEXT ;
}


sub chcontext {
	my $ctx = shift ;

	my $cur_ctx = undef ;
	if ($KERNEL_TYPE == $KERNEL_NEW){
		my $data = pack("II", 0, 0) ;
		$cur_ctx = syscall($NEW_VSERVER_SYSCALL, $NEW_VCMD_S_CONTEXT, $ctx, $data) ;
	}
	elsif ($KERNEL_TYPE == $KERNEL_OLD){
		$cur_ctx = syscall($OLD_S_CONTEXT_SYSCALL, $ctx, 0, 0) ;
	}
	else{
		die("chcontext not supported\n") ;
	}

	if ($cur_ctx == -1){
		die("Can't change to security context $ctx\n") ;
	}
	if ($ctx == -1){
		$ctx = $cur_ctx ;
	}
	$CURRENT_S_CONTEXT = $ctx ;

	return $CURRENT_S_CONTEXT ;
}


sub chbind {
	my @iproot = @_ ;

	my @bin_ips = map {
		my $ip = gethostbyname($_) ;
		(defined($ip) ? $ip : ()) ;
	} @iproot ;
	my @bin_nms = map {
		inet_aton("255.255.255.0") ;
	} @iproot ;

	if (scalar(@bin_ips)){
		my $ipv4root = join(' ', @iproot) ;
		my $rc = undef ;

		if ($KERNEL_TYPE == $KERNEL_NEW){
			my $data = pack("I", -1) . join('', map {($_, inet_aton("255.255.255.0"))} @bin_ips) ;
			$rc = syscall($NEW_VSERVER_SYSCALL, $NEW_VCMD_IPV4ROOT, scalar(@bin_ips), $data) ;
		}
		elsif ($KERNEL_TYPE == $KERNEL_OLD){
			my $rc = undef ;
			if ($OLD_IPV4ROOT_SYSCALL_VERSION eq 'rev3'){
				my $la = join('', @bin_ips) ;
				$rc = syscall($OLD_IPV4ROOT_SYSCALL, $la, scalar(@bin_ips), -1, @bin_nms) ;
			}
			elsif ($OLD_IPV4ROOT_SYSCALL_VERSION eq 'rev2'){
				my $la = join('', @bin_ips) ;
				$rc = syscall($OLD_IPV4ROOT_SYSCALL, $la, scalar(@bin_ips), -1) ;
			}
			elsif ($OLD_IPV4ROOT_SYSCALL_VERSION eq 'rev1'){
				my $la = $bin_ips[0] ;
				$rc = syscall($OLD_IPV4ROOT_SYSCALL, $la, -1) ;
			}
			else {
				my $la = $bin_ips[0] ;
				$rc = syscall($OLD_IPV4ROOT_SYSCALL, $la) ;
			}
		}
		else{
			die("chbind not supported\n") ;
		}

		if ($rc == -1){
			die("Can't set ipv4root to $ipv4root\n") ;
		}

		return $rc ;
	}

	return 0 ;
}


sub vserver_enter {
	my $name = shift ;
	my $ctx = shift ;
	my @iproot = @_ ;

	# We must perform a few system calls to isolate ourselves
	if ((! chroot("/vservers/$name/"))||(! chdir("/"))){
		die("Can't chroot('/vservers/$name/') and then chdir('/'): $!\n") ;
	}

	$ctx = GrabInfo::VServer::chcontext($ctx) ;
	GrabInfo::VServer::chbind(@iproot) ;

	return $ctx ;
}


sub find_vservers {
	my $plugin = shift ;

	my %vservers = () ;
	if ((-d "/vservers/")&&(-d "/etc/vservers/")){
		# There are vservers on this machine
		if (opendir(DIR, "/vservers/")){
			%vservers = map {($_ => {name => $_})} grep {((-d "/vservers/$_")&&(-e "/etc/vservers/$_.conf"))} readdir DIR ;
			closedir(DIR) ;

			foreach my $v (values %vservers){
				$v->{context} = get_vserver_ctx($plugin, $v->{name}) ;
				my $running = 1 ;
				if (($v->{context} == -1)||(! -e "/vservers/$v->{name}/proc/cpuinfo")){
					$running = 0 ;
				}
				$v->{running} = $running ;

				# We need the IP list for that vserver
				my ($prof, @ips) = get_vserver_stuff($plugin, $v->{name}) ;
				$v->{profile} = $prof ;
				$v->{iproot} = \@ips ;
			}
		}
	}

	return \%vservers ;
}


sub get_vserver_ctx {
	my $plugin = shift ;
	my $name = shift ;

	if (-e "/var/run/vservers/$name.ctx"){
		my $lines = $plugin->run_cmd("source /var/run/vservers/$name.ctx 2>&1 && echo \$S_CONTEXT") ;
		return int($lines->[0]) ;
	}

	return -1 ;
}


sub get_vserver_stuff {
	my $plugin = shift ;
	my $name = shift ;

	my $lines = $plugin->run_cmd(
		"source /etc/vservers/$name.conf 2>&1 && echo -e " .
		"\"\$PROFILE\n\$IPROOT\"") ;
	my @ret = () ;
	my ($profile, $addrs) = ($lines->[0], $lines->[1]) ;
	my @addrs = split(/\s+/, $addrs) ;
	foreach my $a (@addrs){
		$a =~ s/^(.*?):// ;
		$a =~ s/\/(.*)$// ;
		push @ret, $a ;		
	}

	return ($profile, @ret) ;
}

						
1 ;
